<?php

/**
 * This file includes \QUI\Control
 */

namespace QUI;

use Exception;
use QUI;
use QUI\Projects\Project;

use function array_keys;
use function array_merge;
use function explode;
use function htmlentities;
use function implode;
use function in_array;
use function is_array;
use function is_numeric;
use function is_object;
use function is_string;
use function json_decode;
use function str_replace;
use function trim;

/**
 * QUI Control
 * PHP counterpart to the \QUI\Control JavaScript class
 *
 * @author www.pcsg.de (Henning Leutz)
 */
class Control extends QDOM implements QUI\Controls\ControlInterface
{
    /**
     * list of css classes
     *
     * @var array
     */
    protected array $cssClasses = [];

    /**
     * @var array
     */
    protected array $cssFiles = [];

    /**
     * @var array
     */
    protected array $styles = [];

    /**
     * JavaScript QUI Require Module name
     * @var string
     */
    protected string $module = '';

    /**
     * @var QUI\Events\Event
     */
    protected Events\Event $Events;

    /**
     * Constructor
     *
     * @param array $attributes
     */
    public function __construct(array $attributes = [])
    {
        $this->Events = new QUI\Events\Event();

        if (!isset($attributes['showTitle']) && !isset($this->attributes['showTitle'])) {
            $attributes['showTitle'] = true;
        }

        if (isset($attributes['styles'])) {
            $this->setStyles($attributes['styles']);
            unset($attributes['styles']);
        }

        if (isset($attributes['class'])) {
            $this->addCSSClass($attributes['class']);
            unset($attributes['class']);
        }

        if (isset($attributes['events'])) {
            $this->addEvents($attributes['events']);
            unset($attributes['events']);
        }

        $this->setAttributes($attributes);
    }

    /**
     * Set the javascript qui module class
     *
     * @param string $module
     */
    protected function setModule(string $module): void
    {
        $this->module = $module;
    }

    /**
     * Return the DOM Node string
     *
     * @return string
     * @throws Exception
     */
    public function create(): string
    {
        try {
            $this->Events->fireEvent('create', [$this]);
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeDebugException($Exception);
        }


        $body = '';

        try {
            $body = $this->getBody();
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }


        $attributes = $this->getAttributes();
        $params = '';

        foreach ($attributes as $key => $value) {
            if (!str_contains($key, 'data-') && $this->isAllowedAttribute($key) === false) {
                continue;
            }

            if (is_object($value) || is_array($value)) {
                continue;
            }

            $key = Utils\Security\Orthos::clear($key);
            $params .= ' ' . $key . '="' . htmlentities($value) . '"';
        }

        // qui class
        $quiClass = '';

        if ($this->getAttribute('qui-class')) {
            $quiClass = 'data-qui="' . $this->getAttribute('qui-class') . '" ';
        }

        $cssClasses = [];

        if ($this->getAttribute('class')) {
            $cssClasses[] = $this->getAttribute('class');
        }

        $cssClasses = array_merge(array_keys($this->cssClasses), $cssClasses);

        if (!empty($cssClasses)) {
            $quiClass .= 'class="' . implode(' ', $cssClasses) . '" ';
        }


        // nodes
        $nodeName = 'div';

        if ($this->getAttribute('nodeName')) {
            $nodeName = $this->getAttribute('nodeName');
        }


        // styles
        if ($this->getAttribute('height')) {
            $this->styles['height'] = $this->cssValueCheck($this->getAttribute('height'));
        }

        if ($this->getAttribute('width')) {
            $this->styles['width'] = $this->cssValueCheck($this->getAttribute('width'));
        }

        // css_inline
        $styles = [];
        $style = '';

        foreach ($this->styles as $property => $value) {
            $property = htmlentities($property);
            $value = htmlentities($value);

            $styles[] = $property . ':' . $value;
        }

        if (!empty($styles)) {
            $style = 'style="' . implode(';', $styles) . '" ';
        }

        try {
            $this->Events->fireEvent('createEnd', [$this]);
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeDebugException($Exception);
        }


        return "<$nodeName $style$quiClass$params>$body</$nodeName>";
    }

    /**
     * Return the inner body of the element
     * Can be overwritten
     *
     * @return string
     * @throws Exception
     */
    public function getBody(): string
    {
        return '';
    }

    /**
     * Set the bound javascript control
     *
     * @param string $control
     */
    public function setJavaScriptControl(string $control): void
    {
        $this->setAttribute('qui-class', $control);
    }

    /**
     * @param string $name
     * @param mixed $value
     */
    public function setJavaScriptControlOption(string $name, mixed $value): void
    {
        $this->setAttribute(
            'data-qui-options-' . $name,
            $value
        );
    }

    /**
     * Add a css class
     *
     * @param array|string $cssClass
     */
    public function addCSSClass(array|string $cssClass): void
    {
        if (empty($cssClass)) {
            return;
        }

        //check if is json
        $json = false;

        if (is_string($cssClass)) {
            $json = json_decode($cssClass, true);
        } elseif (is_array($cssClass)) {
            $json = $cssClass;
        }

        if (is_array($json)) {
            foreach ($json as $cssClass) {
                if (!isset($this->cssClasses[$cssClass])) {
                    $this->cssClasses[$cssClass] = true;
                }
            }

            return;
        }

        $classes = QUI\ControlUtils::clearClassName($cssClass);
        $classes = explode(' ', $classes);

        foreach ($classes as $cssClass) {
            if (!isset($this->cssClasses[$cssClass])) {
                $this->cssClasses[$cssClass] = true;
            }
        }
    }

    /**
     * Remove a css class from the CSS list
     *
     * @param string $cssClass
     */
    public function removeCSSClass(string $cssClass): void
    {
        if (isset($this->cssClasses[$cssClass])) {
            unset($this->cssClasses[$cssClass]);
        }
    }

    /**
     * Add a css file to the control
     *
     * @param string $file
     */
    public function addCSSFile(string $file): void
    {
        $this->cssFiles[] = $file;

        Control\Manager::addCSSFile($file);
    }

    /**
     * Add css files to the control
     *
     * @param array $files
     */
    public function addCSSFiles(array $files): void
    {
        foreach ($files as $file) {
            $this->cssFiles[] = $file;

            Control\Manager::addCSSFile($file);
        }
    }

    /**
     * Return the added css files
     *
     * @return array
     */
    public function getCSSFiles(): array
    {
        return $this->cssFiles;
    }

    /**
     * @param $val
     *
     * @return string
     */
    protected function cssValueCheck($val): string
    {
        $val = trim($val);

        if (empty($val)) {
            return '';
        }

        if (is_numeric($val)) {
            return $val . 'px';
        }

        if (str_contains($val, 'calc(')) {
            return (string)$val;
        }

        $units = [
            'px',
            'cm',
            'mm',
            'mozmm',
            'in',
            'pt',
            'pc',
            'vh',
            'vw',
            'vm',
            'vmin',
            'vmax',
            'rem',
            '%',
            'em',
            'ex',
            'ch',
            'fr',
            'deg',
            'grad',
            'rad',
            's',
            'ms',
            'turns',
            'Hz',
            'kHz'
        ];

        $no = (int)$val;
        $unit = str_replace($no, '', $val);

        if (in_array($unit, $units)) {
            return $no . $unit;
        }

        if (!empty($no) && empty($unit)) {
            return $no . 'px';
        }

        return '';
    }

    /**
     * Set inline style to the control
     *
     * @param string $property - name of the property
     * @param string|int|float $value - value of the property
     */
    public function setStyle(string $property, string|int|float $value): void
    {
        $this->styles[$property] = $value;
    }

    /**
     * Set multiple inline styles to the control
     *
     * @param array $styles
     */
    public function setStyles(array $styles): void
    {
        foreach ($styles as $property => $value) {
            $this->setStyle($property, $value);
        }
    }

    /**
     * Is the html node attribute allowed
     *
     * @param $attribute
     * @return boolean
     */
    protected function isAllowedAttribute($attribute): bool
    {
        $list = [
            'disabled' => true,
            'alt' => true,
            'title' => true,
            'href' => true,
            'target' => true,
            'role' => true
        ];

        return isset($list[$attribute]);
    }

    /**
     * Return the Project
     *
     * @return Project
     *
     * @throws Exception
     */
    protected function getProject(): Projects\Project
    {
        if ($this->getAttribute('Project')) {
            return $this->getAttribute('Project');
        }

        $Project = QUI::getRewrite()->getProject();
        $this->setAttribute('Project', $Project);

        return $Project;
    }

    //region events

    /**
     * @param string $event - The type of event (e.g. 'complete').
     * @param callable $fn - The function to execute.
     */
    public function addEvent(string $event, callable $fn): void
    {
        $this->Events->addEvent($event, $fn);
    }

    /**
     * @param array $events
     */
    public function addEvents(array $events): void
    {
        $this->Events->addEvents($events);
    }

    /**
     * @param string $event - The type of event (e.g. 'complete').
     * @param callable|bool $fn - (optional) The function to remove.
     */
    public function removeEvent(string $event, callable|bool $fn = false): void
    {
        $this->Events->removeEvent($event, $fn);
    }

    /**
     * @param array $events - (optional) If not passed removes all events of all types.
     */
    public function removeEvents(array $events): void
    {
        $this->Events->removeEvents($events);
    }

    //endregion
}
